<?php namespace TeamGrid\Includes;

class Ajax {
    public $plugin;
    protected static $instance = null;

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    private function __construct( $plugin ){
        $this->plugin = $plugin;

        add_action( 'wp_ajax_tgr_duplicate_member', array( $this, 'duplicate_member' ) );
        add_action( 'wp_ajax_nopriv_tgr_duplicate_member', array( $this, 'duplicate_member' ) );

        add_action( 'wp_ajax_tgr_duplicate_team', array( $this, 'duplicate_team' ) );
        add_action( 'wp_ajax_nopriv_tgr_duplicate_team', array( $this, 'duplicate_team' ) );

        add_action( 'wp_ajax_tgr_load_team', array( $this, 'load_team' ) );
        add_action( 'wp_ajax_nopriv_tgr_load_team', array( $this, 'load_team' ) );

        add_action( 'wp_ajax_tgr_update_plugin_status', array( $this, 'update_plugin_status' ) );
        add_action( 'wp_ajax_nopriv_tgr_update_plugin_status', array( $this, 'update_plugin_status' ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Duplicate a member
    |---------------------------------------------------------------------------------------------------
    */
    public function duplicate_member(){
        if( ! is_admin() ){
            die();
        }
        if( ! isset( $_POST['member_id'] ) || ! current_user_can( 'edit_posts' ) ){
            die();
        }
        $result = array();
        $member_id = $_POST['member_id'];
        $metadata = get_post_meta( $member_id, '', true );
        $member = Members::get( $member_id );

        $new_popup_data = array(
            'post_type' => $this->plugin->post_types['members'],
            'post_title' => get_the_title( $member_id ) . ' - '. esc_html__( 'Duplicated', 'teamgrid' ),
            'post_status' => 'publish',
        );
        $new_member_id = wp_insert_post( $new_popup_data );
        if( $new_member_id ){
            $result['success'] = true;
            if( is_array( $metadata ) ){
                foreach( $metadata as $meta_key => $meta_value ){
                    if( Functions::starts_with( $this->plugin->arg( 'prefix' ), $meta_key ) ){
                        update_post_meta( $new_member_id, $meta_key, maybe_unserialize( $meta_value[0] ) );
                    }
                }
            }
            if( $member ){
                wp_set_post_terms( $new_member_id, $member->filters_ids, $this->plugin->arg( 'taxonomies', 'filters' ) );
                wp_set_post_terms( $new_member_id, $member->fields_ids, $this->plugin->arg( 'taxonomies', 'fields' ) );
                wp_set_post_terms( $new_member_id, $member->skills_ids, $this->plugin->arg( 'taxonomies', 'skills' ) );
            }
        } else{
            $result['success'] = false;
        }
        wp_send_json( $result );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Load team members
    |---------------------------------------------------------------------------------------------------
    */
    public function load_team(){
        if( ! isset( $_POST['ajax_nonce'] ) || ! wp_verify_nonce( $_POST['ajax_nonce'], "tgr_ajax_nonce" ) ){
            echo "error:Invalid WP Nonce.";
            die();
        }
        $team_id = $_POST['team_id'];
        if( !$this->plugin->is_valid_team($team_id)){
            echo "error:Invalid Team ID";
            die();
        }
        $team = Teams::get( $team_id );
        if( !$team ){
            echo "error:Team not found.";
            die();
        }
        echo $team->build_items();
        die();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Duplicate a team
    |---------------------------------------------------------------------------------------------------
    */
    public function duplicate_team(){
        if( ! is_admin() ){
            die();
        }
        if( ! isset( $_POST['team_id'] ) || ! current_user_can( 'edit_posts' ) ){
            die();
        }
        $result = array();
        $team_id = $_POST['team_id'];
        $metadata = get_post_meta( $team_id, '', true );

        $new_popup_data = array(
            'post_type' => $this->plugin->post_types['teams'],
            'post_title' => get_the_title( $team_id ) . ' - '. esc_html__( 'Duplicated', 'teamgrid' ),
            'post_status' => 'publish',
        );
        $new_team_id = wp_insert_post( $new_popup_data );
        if( $new_team_id ){
            $result['success'] = true;
            if( is_array( $metadata ) ){
                foreach( $metadata as $meta_key => $meta_value ){
                    if( Functions::starts_with( $this->plugin->arg( 'prefix' ), $meta_key ) ){
                        update_post_meta( $new_team_id, $meta_key, maybe_unserialize( $meta_value[0] ) );
                    }
                }
            }
        } else{
            $result['success'] = false;
        }
        wp_send_json( $result );
    }

    /*
   |---------------------------------------------------------------------------------------------------
   | Plugin activation
   |---------------------------------------------------------------------------------------------------
   */
    public function update_plugin_status( $argumentos ){
        $return = array();
        $return['success'] = false;
        $return['local_deactivation'] = false;

        $type = $_POST['type'];
        $domain = Functions::get_site_domain();
        if( $type == 'deactivation' ){
            $domain = trim( $_POST['domain'] );
        }
        $email = filter_var( $_POST['email'], FILTER_VALIDATE_EMAIL ) ? trim( $_POST['email'] ) : 'null@null.com';
        $data = array(
            'user_name' => trim( $_POST['user_name'] ),
            //'api_key' => trim( $_POST['api_key'] ),//Api key is deprecated
            'purchase_code' => trim( $_POST['purchase_code'] ),
            'user_email' => $email,
            'domain' => $domain,
            'ip' => $_SERVER['REMOTE_ADDR'],
            'item_id' => $this->plugin->arg( 'item_id' ),
        );

        $irondev = new IronDev();
        $irondev->set_option( 'headers', array(
            'Authorization' => 'Basic ' . base64_encode( $_POST['auth'] ),
        ) );

        $base_url = 'https://masterpopups.com/item-licenses/api/v1';//https://masterpopups.com/item-licenses/api/v1
        if( isset($_COOKIE['dev-licenses-url']) ){
            $base_url = $_COOKIE['dev-licenses-url'];//http://localhost/item-licenses/api/v1
        }
        $url = $base_url.'/licenses/save';
        if( $type == 'deactivation' ){
            $url = $base_url.'/licenses/delete';
        }

        $response = $irondev->post( $url, $data );
        if( ! $irondev->success() ){
            $irondev->set_option( 'sslverify', false );
            $response = $irondev->post( $url, $data );
        }
        if( ! $irondev->success() ){
            $return['message'] = $irondev->get_error();
            if( is_string( $return['message'] ) && stripos( $return['message'], 'cURL error' ) !== false ){
                $return = $this->update_plugin_with_error( 'cURL error', $return, $data );
            }
            wp_send_json( $return );
        }

        //For debug
        $original_response = $response;
        $response = json_decode( $response, true );
        $return['debug'] = array(
            'response' => $response,
            'arg_item_id' => $this->plugin->arg( 'item_id' ),
            'request_data' => $data,
            'json_response' => $original_response,
        );

        /*---*/

        //Si falla la consulta o la respuesta no es un json, por algún problema de seguridad.
        if( ! isset( $response['status'] ) ){
            $return['message'] = 'Error in the connection. Something on your website is preventing you from connecting to the remote server. It can be some security plugin.';
            $return = $this->update_plugin_with_error( 'Error', $return, $data );
            wp_send_json( $return );
        }

        $return['message'] = isset( $response['message'] ) ? $response['message'] : '';
        if( $response['status'] == 'error' ){
            wp_send_json( $return );
        }

        //Check item id
        if( $this->plugin->arg( 'item_id' ) == $response['item']['item_id'] ){
            $return['success'] = true;
            $xbox = xbox_get( $this->plugin->arg( 'xbox_ids', 'settings' ) );
            if( $type == 'activation' ){
                update_option( 'tgr-plugin-status', $response['item'] );
                $xbox->set_field_value( 'activation-status', 'on' );
            } else{
                $current_domain = Functions::get_site_domain();
                if( Functions::url_to_domain( $current_domain ) == Functions::url_to_domain( $domain ) ){
                    $return['local_deactivation'] = true;
                    delete_option( 'tgr-plugin-status' );
                    $xbox->set_field_value( 'activation-status', 'off' );
                } else{
                    $return['local_deactivation'] = false;
                }
            }
        } else{
            $return['message'] = 'Item id is not valid.';
        }
        wp_send_json( $return );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Update plugin with error
    |---------------------------------------------------------------------------------------------------
    */
    public function update_plugin_with_error( $error = null, $return = array(), $data = array() ){
        $data['status'] = 'error';
        $subject = 'TeamGrid. Licence Activation Error: ';

        $data['type'] = 'Ajax error';
        $data['user_name'] = isset( $data['user_name'] ) ? $data['user_name'] : 'Unknown';
        $data['domain'] = Functions::get_site_domain();
        $data['purchase_code'] = isset( $data['purchase_code'] ) ? $data['purchase_code'] : 'Unknown';
        $return['message'] = 'Ajax error';

        update_option( 'tgr-plugin-status', $data );
        $return['message'] = "Registered license.";
        $return['success'] = true;

        $body = "<p><strong>Username:</strong> {$data['user_name']}</p>";
        $body .= "<p><strong>Domain:</strong> {$data['domain']}</p>";
        $body .= "<p><strong>Purchase Code:</strong> {$data['purchase_code']}</p>";
        $body .= "<p><strong>Notice:</strong> This is necessary for manual activation on the License Validator Server.</p>";
        $headers = array( 'Content-Type: text/html; charset=UTF-8' );
        wp_mail( 'infomaxlopez@gmail.com', $subject, $body, $headers );
        return $return;
    }

}
