<?php namespace TeamGrid\Includes;

abstract class BaseOptions {
    public $id = 0;
    public $ID = 0;
    public $title = '';
    protected $status = 'on';
    protected $options = array();
    public $initial_options = array();
    public $xbox = null;

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    protected function __construct( $options = array(), $defaults = array() ){
        $this->xbox = xbox_get( $this->xbox_id );

        $id = ! empty( $options['id'] ) ? $options['id'] : 0;
        if( $this->set_object_id( $id ) ){
            $this->title = html_entity_decode( get_the_title( $this->id ) );
        }

        $this->set_defaults( $defaults );
        $this->set_options( $options, $defaults );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_options(){
        return $this->options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add initial options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_defaults( $defaults = array() ){
        $this->initial_options = $defaults;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Set all options
    |---------------------------------------------------------------------------------------------------
    */
    abstract protected function set_options( $options = array() );

    /*
    |---------------------------------------------------------------------------------------------------
    | Set current object ID
    |---------------------------------------------------------------------------------------------------
    */
    public function set_object_id( $id = 0 ){
        if( $id ){
            $this->id = $id;
            $this->ID = $this->id;
            return true;
        } else if( Functions::is_post_page( 'new' ) ){
            $this->id = 0;
            $this->ID = 0;
            return false;
        } else {
            $this->id = Functions::post_id();
            $this->ID = $this->id;
            return ! empty( $this->id );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Access to any method, avoid errors when calling non-existent methods
    |---------------------------------------------------------------------------------------------------
    */
    public function __call( $name, $arguments ){
        if( Functions::starts_with( 'set_', $name ) && strlen( $name ) > 4 ){
            $property = substr( $name, 4 );
            if( property_exists( $this, $property ) && isset( $arguments[0] ) ){
                $this->$property = $arguments[0];
                return $this->$property;
            }
            return null;
        } else if( Functions::starts_with( 'get_', $name ) && strlen( $name ) > 4 ){
            $property = substr( $name, 4 );
            if( property_exists( $this, $property ) ){
                return $this->$property;
            }
            return null;
        } else if( property_exists( $this, $name ) ){
            return $this->$name;
        } else{
            return $this->option( $name );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Change the value to an option
    |---------------------------------------------------------------------------------------------------
    */
    public function set_option( $option_name, $value = null ){
        $option_name = $this->get_option_name( $option_name );
        if( isset( $this->options[$option_name] ) && ! is_null( $value ) ){
            $this->options[$option_name] = $value;
        }
        return $this->options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the value of an option
    |---------------------------------------------------------------------------------------------------
    */
    public function get_option( $option_name = '', $default_value = null ){
        return $this->option( $option_name, $default_value );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the value of an option
    |---------------------------------------------------------------------------------------------------
    */
    public function option( $option_name = '', $default_value = null ){
        $option_name = $this->get_option_name( $option_name );
        $value = null;
        if( isset( $this->options[$option_name] ) ){
            $value = $this->options[$option_name];
        } else {
            $this->set_option_to( $this->options, $option_name, $default_value );
            $value = $this->options[$option_name];
        }
        return $value;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Gets the actual name of the option
    |---------------------------------------------------------------------------------------------------
    */
    public function get_option_name( $name ){
        if( $this->is_metabox ){
            if( ! Functions::starts_with( TGR_PREFIX, $name ) ){
                return TGR_PREFIX . $name;
            }
        }
        return $name;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add new options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_new_default_options( $options = array() ){
        foreach( $options as $name => $value ){
            $this->set_option_to( $this->options, $name, $value );
        }
        return $this->options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add option to option array
    |---------------------------------------------------------------------------------------------------
    */
    public function set_option_to( &$array = array(), $option_name = '', $default = '' ){
        //Prefix is important for the import to work.
        $option_name = $this->get_option_name( $option_name );
        if( $this->xbox ){
            if( strpos( get_class( $this->xbox ), 'Metabox') !== false  ){
                $array[$option_name] = $this->xbox->get_field_value( $option_name, $this->id, $default );
            } else {
                $array[$option_name] = $this->xbox->get_field_value( $option_name, $default );
            }
        } else {
            //xbox does not exist when the option is called within the same options
            $value = $this->get_field_value( $option_name, $this->id, $default );
            $array[$option_name] = $value;
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Helper function to get the value of an option
    |---------------------------------------------------------------------------------------------------
    */
    public function get_field_value( $option_name, $post_id, $default = null ){
        $value = null;
        if( $this->is_metabox ){
            $value = get_post_meta( $post_id, $option_name, true );
        } else if( $this->is_settings_page ) {
            $options = get_option( $this->xbox_id );
            //When the options have not yet been saved for the first time
            if( $options == false ){
                $options = Settings::$defaults;
            }
            if( isset( $options[$option_name] ) ){
                $value = $options[$option_name];
            }
        }
        if( Functions::is_empty( $value ) ){
            $value = $default;
        }
        return $value;
    }

}
