<?php namespace TeamGrid\Includes;

class Functions {

    /*
	|---------------------------------------------------------------------------------------------------
	| Gets the current plugin instance
	|---------------------------------------------------------------------------------------------------
	*/
    public static function get_plugin_instance(){
        return TeamGrid::get_instance();
    }
    /*
	|---------------------------------------------------------------------------------------------------
	| Current post ID
	|---------------------------------------------------------------------------------------------------
	*/
    public static function get_the_ID(){
        $post = get_post();
        return ! empty( $post ) ? $post->ID : false;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get post id
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_id(){
        return self::post_id();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get post id
    |---------------------------------------------------------------------------------------------------
    */
    public static function post_id(){
        $post_id = 0;
        if( ! is_admin() ){
            $post_id = self::get_the_ID();
        } else{
            if( Functions::is_post_page( 'new' ) ){
                return 0;
            }
            if( ! $post_id ){
                $post_id = isset( $_REQUEST['post'] ) ? $_REQUEST['post'] : $post_id;
            }
            if( ! $post_id ){
                $post_id = isset( $_GET['post'] ) ? $_GET['post'] : $post_id;
            }
            if( ! $post_id ){
                $post_id = isset( $GLOBALS['post']->ID ) ? $GLOBALS['post']->ID : 0;
            }
        }

        if( is_array( $post_id ) ){
            return 0;
        }
        return $post_id;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if a string starts with a specific character
    |---------------------------------------------------------------------------------------------------
    */
    public static function starts_with( $needle, $haystack, $case_sensitive = false ){
        if( strlen( $needle ) == 0 || strlen( $haystack ) == 0 ){
            return false;
        }
        return substr_compare( $haystack, $needle, 0, strlen( $needle ), ! $case_sensitive ) === 0;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if a string ends with a specific character
    |---------------------------------------------------------------------------------------------------
    */
    public static function ends_with( $needle, $haystack, $case_sensitive = false ){
        $offset = strlen( $haystack ) - strlen( $needle );
        if( strlen( $needle ) == 0 || strlen( $haystack ) == 0 || $offset >= strlen( $haystack ) ){
            return false;
        }
        return substr_compare( $haystack, $needle, $offset, strlen( $needle ), ! $case_sensitive ) === 0;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Sort an array
    |---------------------------------------------------------------------------------------------------
    */
    public static function sort( &$array = array(), $sort = 'asc' ){
        if( strtolower( $sort ) == 'asc' ){
            ksort( $array );
        } elseif( strtolower( $sort ) == 'desc' ){
            krsort( $array );
        }
        return $array;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if a variable is empty
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_empty( $value = '' ){
        if( is_array( $value ) ){
            $value = array_filter( $value );
            if( empty( $value ) ){
                return true;
            }
            return false;
        } else if( is_numeric( $value ) ){
            return false;
        } else if( empty( $value ) ){
            return true;
        } else{
            return false;
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get the id of an attachment through a url
    |---------------------------------------------------------------------------------------------------
    | https://wpscholar.com/blog/get-attachment-id-from-wp-image-url/
    */
    public static function get_attachment_id_by_url( $url ){
        $attachment_id = 0;
        $dir = wp_upload_dir();
        $url = str_replace( array('https', 'http'), '', $url );
        $dir['baseurl'] = str_replace( array('https', 'http'), '', $dir['baseurl'] );
        if( false !== strpos( $url, $dir['baseurl'] . '/' ) ){ // Is URL in uploads directory?
            $file = basename( self::get_protocol().$url );
            $query_args = array(
                'post_type' => 'attachment',
                'post_status' => 'inherit',
                'fields' => 'ids',
                'meta_query' => array(
                    array(
                        'value' => $file,
                        'compare' => 'LIKE',
                        'key' => '_wp_attachment_metadata',
                    ),
                )
            );
            $query = new \WP_Query( $query_args );
            if( $query->have_posts() ){
                foreach( $query->posts as $post_id ){
                    $meta = wp_get_attachment_metadata( $post_id );
                    $original_file = basename( $meta['file'] );
                    $cropped_image_files = wp_list_pluck( $meta['sizes'], 'file' );
                    if( $original_file === $file || in_array( $file, $cropped_image_files ) ){
                        $attachment_id = $post_id;
                        break;
                    }
                }
            }
        }
        return $attachment_id;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if we are on an options page. It only works with pages within admin.php?page=*
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_admin_page(){
        global $pagenow;
        if( ! is_admin() ){
            return false;
        }
        return $pagenow == 'admin.php';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if we are on an options page
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_post_page( $page = '' ){
        global $pagenow;
        if( ! is_admin() ){
            return false;
        }
        if( $page == 'edit' ){
            return in_array( $pagenow, array( 'post.php' ) );
        } elseif( $page == 'new' ){
            return in_array( $pagenow, array( 'post-new.php' ) );
        }
        return in_array( $pagenow, array( 'post.php', 'post-new.php' ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if we are on an admin page of a post_type
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_admin_page_post_type( $post_type = '' ){
        $screen = get_current_screen();
        if( is_admin() && $screen && isset( $screen->parent_file ) ){
            return $screen->parent_file == 'edit.php?post_type=' . $post_type;
        }
        return false;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if we are on the page for editing or creating a specific post_type
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_editing_post_type( $post_type = '' ){
        if( ! Functions::is_post_page() || Functions::get_current_post_type() != $post_type ){
            return false;
        }
        return true;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if we are on the plugin settings page
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_page_plugin_settings(){
        $plugin = self::get_plugin_instance();
        return isset( $_GET['page'] ) && $_GET['page'] == $plugin->arg( 'xbox_ids', 'settings' );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Admin url of a post type
    |---------------------------------------------------------------------------------------------------
    */
    public static function post_type_url( $post_type, $action = 'edit', $query = array() ){
        $action = $action == 'new' ? 'post-new.php?post_type=' : 'edit.php?post_type=';
        $end_url = is_array( $query ) && ! empty( $query ) ? '&' . http_build_query( $query ) : '';
        return get_admin_url() . $action . $post_type . $end_url;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Gets the current post type
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_current_post_type( $post_id = null ){
        global $post, $typenow, $current_screen;
        if( $typenow ){
            return $typenow;
        } elseif( $current_screen && $current_screen->post_type ){
            return $current_screen->post_type;
        } elseif( isset( $_REQUEST['post_type'] ) ){
            return sanitize_key( $_REQUEST['post_type'] );
        } elseif( isset( $_GET['post'] ) ){
            return get_post_type( $_GET['post'] );
        } else if( $post_id && $post_type = get_post_type( $post_id ) ){
            return $post_type;
        } else if( $post && $post->post_type ){
            return $post->post_type;
        }
        return null;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if we are on the main page
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_homepage(){
        $is_homepage = false;
        if( is_front_page() && is_home() ){
            $is_homepage = true; //Default homepage
        } elseif( is_front_page() ){
            $is_homepage = true; //Static homepage
        } elseif( is_home() ){
            $is_homepage = true; //Blog page
        } elseif( $_SERVER["REQUEST_URI"] == '/' ){
            $is_homepage = true;
        }
        return $is_homepage;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if it is localhost
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_localhost(){
        return in_array( strtolower( $_SERVER['SERVER_NAME'] ), array( 'localhost', '127.0.0.1' ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Return HTTP Protocol
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_protocol(){
        return ( ! empty( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443 ) ? "https" : "http";
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get current url
    |---------------------------------------------------------------------------------------------------
    */
    public static function current_url( $in_ajax = null, $end_slash = true ){
        global $wp;
        if( $in_ajax === true ){
            $current_url = $_SERVER['HTTP_REFERER'];
        } else if( $in_ajax === false ){
            $current_url = self::get_protocol() . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
        } else{
            $current_url = home_url( add_query_arg( array(), $wp->request ) );
        }
        if( $end_slash ){
            return trailingslashit( $current_url );
        } else{
            return untrailingslashit( $current_url );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the domain of a url
    |---------------------------------------------------------------------------------------------------
    */
    public static function url_to_domain( $url, $length = 60 ){
        $url = trim( $url );
        $host = parse_url( $url, PHP_URL_HOST );
        //If the URL can't be parsed, use the original URL
        if( ! $host ){
            $host = $url;
        }
        //Remove www.
        if( substr( $host, 0, 4 ) == 'www.' ){
            $host = substr( $host, 4 );
        }
        //Limit the domain length
        if( strlen( $host ) > $length ){
            $host = substr( $host, 0, $length - 3 ) . '...';
        }
        return $host;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Current site domain
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_site_domain(){
        return empty( $_SERVER['SERVER_NAME'] ) ? $_SERVER['HTTP_HOST'] : $_SERVER['SERVER_NAME'];
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Join two arrays, allow to exclude keys and join values of the same key
    |---------------------------------------------------------------------------------------------------
    */
    public static function nice_array_merge( $attrs = array(), $new_attrs = array(), $exclude_keys = array(), $join_keys = array() ){
        $join_array_keys = isset( $join_keys[0] ) ? $join_keys : array_keys( $join_keys );

        foreach( $new_attrs as $key => $val ){
            if( in_array( $key, $exclude_keys ) ){
                continue;
            }
            if( isset( $attrs[$key] ) && in_array( $key, $join_array_keys ) ){
                $separator = isset( $join_keys[0] ) ? ' ' : $join_keys[$key];
                $attrs[$key] = $attrs[$key] . $separator . $val;
            } else{
                $attrs[$key] = $val;
            }
        }
        return $attrs;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build a valid url from Google fonts
    |---------------------------------------------------------------------------------------------------
    */
    public static function url_google_fonts( $google_fonts = array(), $exclude = array() ){
        $fonts = array();
        foreach( $google_fonts as $font_family ){
            if( ! in_array( $font_family, $exclude ) ){
                $weights = '100,200,300,400,500,600,700,800,900';
                $font = str_replace( ' ', '+', $font_family ) . ':' . $weights;
                $fonts[] = $font;
            }
        }
        $fonts = implode( '|', $fonts );
        if( empty( $fonts ) ){
            return "";
        }
        return "//fonts.googleapis.com/css?family={$fonts}&amp;subset=latin,latin-ext,greek,greek-ext,cyrillic,cyrillic-ext,vietnamese";
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Validate an email
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_email( $email = '' ){
        return filter_var( $email, FILTER_VALIDATE_EMAIL );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Parse text content
	|---------------------------------------------------------------------------------------------------
	*/
    public static function parse_content( $content ){
        $content = do_shortcode( $content );
        $content = wpautop( $content );
        return $content;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Truncate string. https://stackoverflow.com/questions/16583676/shorten-text-without-splitting-words-or-breaking-html-tags
	|---------------------------------------------------------------------------------------------------
	*/
    /**
     * Truncates text.
     *
     * Cuts a string to the length of $length and replaces the last characters
     * with the ending if the text is longer than length.
     *
     * ### Options:
     *
     * - `ending` Will be used as Ending and appended to the trimmed string
     * - `exact` If false, $text will not be cut mid-word
     * - `html` If true, HTML tags would be handled correctly
     *
     * @param string  $text String to truncate.
     * @param integer $length Length of returned string, including ellipsis.
     * @param array $options An array of html attributes and options.
     * @return string Trimmed string.
     * @access public
     * @link http://book.cakephp.org/view/1469/Text#truncate-1625
     */
    public static function truncate($text, $length = 100, $options = array()) {
        $default = array(
            'ending' => '...', 'exact' => true, 'html' => false
        );
        $options = array_merge($default, $options);
        $length = $length + strlen( $options['ending'] );

        if ($options['html']) {
            if (mb_strlen(preg_replace('/<.*?>/', '', $text)) <= $length) {
                return $text;
            }
            $totalLength = mb_strlen(strip_tags($options['ending']));
            $openTags = array();
            $truncate = '';

            preg_match_all('/(<\/?([\w+]+)[^>]*>)?([^<>]*)/', $text, $tags, PREG_SET_ORDER);
            foreach ($tags as $tag) {
                if (!preg_match('/img|br|input|hr|area|base|basefont|col|frame|isindex|link|meta|param/s', $tag[2])) {
                    if (preg_match('/<[\w]+[^>]*>/s', $tag[0])) {
                        array_unshift($openTags, $tag[2]);
                    } else if (preg_match('/<\/([\w]+)[^>]*>/s', $tag[0], $closeTag)) {
                        $pos = array_search($closeTag[1], $openTags);
                        if ($pos !== false) {
                            array_splice($openTags, $pos, 1);
                        }
                    }
                }
                $truncate .= $tag[1];

                $contentLength = mb_strlen(preg_replace('/&[0-9a-z]{2,8};|&#[0-9]{1,7};|&#x[0-9a-f]{1,6};/i', ' ', $tag[3]));
                if ($contentLength + $totalLength > $length) {
                    $left = $length - $totalLength;
                    $entitiesLength = 0;
                    if (preg_match_all('/&[0-9a-z]{2,8};|&#[0-9]{1,7};|&#x[0-9a-f]{1,6};/i', $tag[3], $entities, PREG_OFFSET_CAPTURE)) {
                        foreach ($entities[0] as $entity) {
                            if ($entity[1] + 1 - $entitiesLength <= $left) {
                                $left--;
                                $entitiesLength += mb_strlen($entity[0]);
                            } else {
                                break;
                            }
                        }
                    }

                    $truncate .= mb_substr($tag[3], 0 , $left + $entitiesLength);
                    break;
                } else {
                    $truncate .= $tag[3];
                    $totalLength += $contentLength;
                }
                if ($totalLength >= $length) {
                    break;
                }
            }
        } else {
            if (mb_strlen($text) <= $length) {
                return $text;
            } else {
                $truncate = mb_substr($text, 0, $length - mb_strlen($options['ending']));
            }
        }
        if (!$options['exact']) {
            $spacepos = mb_strrpos($truncate, ' ');
            if (isset($spacepos)) {
                if ($options['html']) {
                    $bits = mb_substr($truncate, $spacepos);
                    preg_match_all('/<\/([a-z]+)>/', $bits, $droppedTags, PREG_SET_ORDER);
                    if (!empty($droppedTags)) {
                        foreach ($droppedTags as $closingTag) {
                            if (!in_array($closingTag[1], $openTags)) {
                                array_unshift($openTags, $closingTag[1]);
                            }
                        }
                    }
                }
                $truncate = mb_substr($truncate, 0, $spacepos);
            }
        }
        $truncate .= $options['ending'];

        if ($options['html']) {
            foreach ($openTags as $tag) {
                $truncate .= '</'.$tag.'>';
            }
        }

        return $truncate;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the names of image sizes configured in wordpress
    | https://developer.wordpress.org/reference/functions/get_intermediate_image_sizes/
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_image_sizes( $size = '' ){
        global $_wp_additional_image_sizes;
        if( ! $_wp_additional_image_sizes ){
            $_wp_additional_image_sizes = array();
        }
        $wp_additional_image_sizes = $_wp_additional_image_sizes;

        $sizes = array();
        $get_intermediate_image_sizes = get_intermediate_image_sizes();

        // Create the full array with sizes and crop info
        foreach( $get_intermediate_image_sizes as $_size ) {
            if ( in_array( $_size, array( 'thumbnail', 'medium', 'large' ) ) ) {
                $sizes[ $_size ]['width'] = get_option( $_size . '_size_w' );
                $sizes[ $_size ]['height'] = get_option( $_size . '_size_h' );
                $sizes[ $_size ]['crop'] = (bool) get_option( $_size . '_crop' );
            } elseif ( isset( $wp_additional_image_sizes[ $_size ] ) ) {
                $sizes[ $_size ] = array(
                    'width' => $wp_additional_image_sizes[ $_size ]['width'],
                    'height' => $wp_additional_image_sizes[ $_size ]['height'],
                    'crop' =>  $wp_additional_image_sizes[ $_size ]['crop']
                );
            }
        }
        // Get only 1 size if found
        if ( $size ) {
            if( isset( $sizes[ $size ] ) ) {
                return $sizes[ $size ];
            } else {
                return false;
            }
        }
        return $sizes;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get user roles, $capabilities = array('edit_posts), $exclude = array('administrator', 'editor')
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_user_roles( $capabilities = array(), $exclude = array() ){
        global $wp_roles;
        $roles = array();
        if( isset( $wp_roles->roles, $wp_roles->role_names ) && is_array( $wp_roles->roles ) ){
            if( empty( $capabilities ) ){
                $roles = $wp_roles->role_names;
            } else{
                foreach( $wp_roles->roles as $role_key => $role_data ){
                    $array_keys = array_keys( $role_data['capabilities'] );
                    foreach( $capabilities as $cap ){
                        if( in_array( $cap, $array_keys ) ){
                            $roles[$role_key] = $role_data['name'];
                        }
                    }
                }
            }
        }
        if( ! empty( $exclude ) ){
            foreach( $roles as $role_key => $role ){
                if( in_array( $role_key, $exclude ) ){
                    unset( $roles[$role_key] );
                }
            }
        }
        return $roles;
    }


}
