<?php namespace TeamGrid\Includes;


class SettingsEditor {
    protected static $instance = null;
    public $plugin;
    public $post_types = array();
    public $xbox_ids = array();
    public $path = array();

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    private function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->post_types = $this->plugin->arg( 'post_types' );
        $this->xbox_ids = $this->plugin->arg( 'xbox_ids' );
        $this->path = trailingslashit( TGR_OPTIONS_DIR . 'general-settings' );

        $this->hooks();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Plugin hooks
    |---------------------------------------------------------------------------------------------------
    */
    private function hooks(){
        add_action( 'xbox_init', array( $this, 'create_metabox' ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Crea metabox
    |---------------------------------------------------------------------------------------------------
    */
    public function create_metabox( $hook ){
        $options = array(
            'id' => $this->xbox_ids['settings'],
            'title' => esc_html__( 'TeamGrid Settings', 'teamgrid' ),
            'menu_title' => esc_html__( 'Settings', 'teamgrid' ),
            'parent' => $this->plugin->main_menu_item,
            'capability' => 'manage_options',
            'class' => 'teamgrid',
            'header' => array(
                'desc' => '',
                'icon' => '',
                'submit-buttons-sticky' => true,
            ),
            'footer' => $this->plugin->arg( 'name' ).' v'.TGR_VERSION,
        );
        $xbox = xbox_new_admin_page( $options );
        $this->add_options( $xbox );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add options
    |---------------------------------------------------------------------------------------------------
    */
    private function add_options( $xbox ){
        $defaults = Settings::$defaults;
        $id_main_tab = 'main-tab';
        $items_main_tab = array(
            'general' => '<i class="xbox-icon xbox-icon-object-group"></i>'.esc_html__( 'General', 'teamgrid' ),
            'activation' => '<i class="xbox-icon xbox-icon-key"></i>'.esc_html__( 'Plugin Activation', 'teamgrid' ),
            'custom-css' => '<i class="xbox-icon xbox-icon-paint-brush"></i>'.esc_html__( 'Custom CSS', 'teamgrid' ),
            'custom-js' => '<i class="xbox-icon xbox-icon-code"></i>'.esc_html__( 'Custom JS', 'teamgrid' ),
            'head-footer-scripts' => '<i class="xbox-icon xbox-icon-code"></i>'.esc_html__( 'Head & Footer Scripts', 'teamgrid' ),
        );
        $items_main_tab = apply_filters('teamgrid_settings_tab_items', $items_main_tab, $id_main_tab );

        $xbox->add_main_tab(array(
            'name' => 'Main tab',
            'id' => 'main-tab',
            'items' => $items_main_tab,
        ));

        $xbox->open_tab_item('general');
        include $this->path . 'general/general-tab.php';
        $xbox->close_tab_item('general');

        $xbox->open_tab_item('activation');
        include $this->path . 'activation/activation.php';
        $xbox->close_tab_item('activation');

        $xbox->open_tab_item('custom-css');
        include $this->path . 'custom-css/custom-css.php';
        $xbox->close_tab_item('custom-css');

        $xbox->open_tab_item('custom-js');
        include $this->path . 'custom-js/custom-js.php';
        $xbox->close_tab_item('custom-js');

        $xbox->open_tab_item('head-footer-scripts');
        include $this->path.'head-footer-scripts.php';
        $xbox->close_tab_item('head-footer-scripts');

        $xbox = apply_filters( 'teamgrid_settings_tab_fields', $xbox );

        $xbox->close_tab('main-tab');
    }
}