<?php namespace TeamGrid\Includes;

use Xbox\Includes\CSS;

class Settings extends BaseOptions {
    private static $instance = null;
    public $plugin = null;

    public static $defaults = array();
    protected $xbox_id = '';
    protected $is_metabox = false;
    protected $is_settings_page = true;

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    protected function __construct( $plugin = null ){
        $this->plugin = $plugin;
        $this->xbox_id = $this->plugin->arg( 'xbox_ids', 'settings' );
        self::$defaults = include TGR_OPTIONS_DIR . 'general-settings/defaults.php';
        $this->hooks();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin hooks
	|---------------------------------------------------------------------------------------------------
	*/
    private function hooks(){
        //The priority must be higher than the creation of the xboxs
        add_action( 'xbox_init', array( $this, 'load_general_settings' ), TGR_XBOX_INIT_PRIORITY + 1 );

        $wp_head_priority = $this->option('header-scripts-priority', 10 );
        $wp_footer_priority = $this->option('footer-scripts-priority', 10 );

        add_action( 'wp_head', array( $this, 'load_resources_in_header' ), $wp_head_priority );
        add_action( 'admin_head', array( $this, 'load_resources_in_header' ), $wp_head_priority );
        add_action( 'wp_footer', array( $this, 'load_resources_in_footer' ), $wp_footer_priority );
        add_action( 'admin_footer', array( $this, 'load_resources_in_footer' ), $wp_footer_priority );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Load default settings
    |---------------------------------------------------------------------------------------------------
    */
    public function load_general_settings(){
        $options = array();
        parent::__construct( $options, self::$defaults );
        do_action( 'teamgrid_general_settings_init', $this->plugin, $this );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Set the default options
    |---------------------------------------------------------------------------------------------------
    */
    protected function set_options( $initial_options = array() ){
        $options = array();
        foreach( self::$defaults as $option_name => $option_value ){
            $this->set_option_to( $options, $option_name, $option_value );
        }
        $this->options = wp_parse_args( $initial_options, $options );
        return $this->options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the value of an option
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_value( $option_name = '', $default_value = null ){
        $plugin = Functions::get_plugin_instance();
        $settings_id = $plugin::$args['xbox_ids']['settings'];
        $options = get_option( $settings_id );
        if( is_array( $options ) && isset( $options[$option_name] ) ){
            return $options[$option_name];
        } elseif( $default_value !== null ){
            return $default_value;
        }
        if( isset( self::$defaults[$option_name] ) ){
            return self::$defaults[$option_name];
        }
        return null;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get the image sizes
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_image_sizes(){
        $image_size_1 = array(
            'name' => self::get_value('img-resizing-1-name'),
            'width' => self::get_value('img-resizing-1-width'),
            'height' => self::get_value('img-resizing-1-height'),
        );
        $image_size_2 = array(
            'name' => self::get_value('img-resizing-2-name'),
            'width' => self::get_value('img-resizing-2-width'),
            'height' => self::get_value('img-resizing-2-height'),
        );
        $image_size_3 = array(
            'name' => self::get_value('img-resizing-3-name'),
            'width' => self::get_value('img-resizing-3-width'),
            'height' => self::get_value('img-resizing-3-height'),
        );
        return array(
            '1' => $image_size_1,
            '2' => $image_size_2,
            '3' => $image_size_3,
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Loading resources in header
    |---------------------------------------------------------------------------------------------------
    */
    public function load_resources_in_header(){
        $version = esc_attr( $this->plugin->arg( 'version' ) );
        echo "\n\n<!-- TGR:TeamGrid v$version -->";
        echo "\n\n<!-- TGR:TeamGrid:Header:Settings -->";
        echo "\n<style>";
        echo "\n/* Custom CSSs*/\n";
        echo wp_strip_all_tags( $this->option( 'custom-css' ) );
        echo wp_strip_all_tags( $this->build_general_styles() );
        echo "\n</style>";
        /*
         * The setting below is not sanitized. In order to be able to take advantage of this,
		 * a user would have to gain access to the database,
		 * in which case this is the least of your worries.
         */
        echo "\n".$this->option( 'header-scripts' );

        echo "\n<!-- TGR:TeamGrid:Header:Settings End -->\n\n";
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Loading resources in header
    |---------------------------------------------------------------------------------------------------
    */
    public function build_general_styles(){
        $style = "";

        //Social icons
        $iconSize = (int) $this->option( 'social-icons-icon-size' );
        $iconFontSize = round( $iconSize / 2.5 );

        $css = new CSS( ".tgr-member-social ul li a" );
        $css->prop( 'font-size', CSS::number( $iconFontSize, 'px' ) );
        $css->prop( 'color', $this->option( 'social-icons-color' ) );
        $css->prop( 'border-color', $this->option( 'social-icons-color' ) );
        $css->prop( 'border-radius', CSS::number( $this->option( 'social-icons-border-radius' ), 'px' ) );
        $css->prop( 'width', CSS::number( $iconSize, 'px' ) );
        $css->prop( 'height', CSS::number( $iconSize, 'px' ) );
        $css->prop( 'line-height', CSS::number( $iconSize - 1, 'px' ) );
        $style .= $css->build_css();

        $css = new CSS( ".tgr-member-social ul li a:hover" );
        $css->prop( 'color', $this->option( 'social-icons-color-hover' ) );
        $css->prop( 'border-color', $this->option( 'social-icons-color-hover' ) );
        $style .= $css->build_css();

        //Skills
        $css = new CSS( ".tgr-member-skills .tgr-skill-bg" );
        $css->prop( 'background', $this->option( 'skills-bg-color' ) );
        $style .= $css->build_css();

        $css = new CSS( ".tgr-member-skills .tgr-skill-percent" );
        $css->prop( 'background', $this->option( 'skills-bg-percent' ) );
        $style .= $css->build_css();

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Loading resources in the footer
    |---------------------------------------------------------------------------------------------------
    */
    public function load_resources_in_footer(){
        echo "\n\n<!-- TGR:TeamGrid:Footer:Settings -->";
        echo "\n<script type='text/javascript'>";
        echo "\n/* Custom JS */\n";
        /*
         * The setting below is not sanitized. In order to be able to take advantage of this,
		 * a user would have to gain access to the database,
		 * in which case this is the least of your worries.
         */
        echo $this->option( 'custom-javascript' );
        echo "\n</script>";
        /*
         * The setting below is not sanitized. In order to be able to take advantage of this,
		 * a user would have to gain access to the database,
		 * in which case this is the least of your worries.
         */
        echo "\n".$this->option( 'footer-scripts' );
        echo "\n<!-- TGR:TeamGrid:Footer:Settings End -->\n\n";
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | plugin_status
    |---------------------------------------------------------------------------------------------------
    */
    public static function plugin_status(){
        $option = get_option( 'tgr-plugin-status' );
        if( $option && is_array( $option ) ){
            return isset( $option['purchase_code'], $option['user_name'] );
        }
        return false;
    }

    public static function plugin_status_message( $url ){
        $header = esc_html__( 'License activation required.', 'teamgrid' );
        $message = sprintf( esc_html__( 'Please activate your license from %shere%s. Plugin Settings > Plugin Activation', 'teamgrid' ), '<a href="' . esc_url( $url ) . '" target="_blank">', '</a>' );
        return '<div class="tgr-message tgr-message-warning tgr-icon-message">
			        <header>' . $header . '</header>
			        <p>' . $message . '</p>
	            </div>';
    }
}
