<?php namespace TeamGrid\Includes;

use Xbox\Includes\CSS;

class TeamOptions extends BaseOptions {
    protected $xbox_id = '';
    protected $is_metabox = true;
    protected $is_settings_page = false;

    public static $layouts = array();


    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    protected function __construct( $options = array(), $defaults = array() ){
        self::$layouts = Assets::get_all_layouts();
        $this->xbox_id = $this->plugin->arg( 'xbox_ids', 'team-editor' );
        parent::__construct( $options, $defaults );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Sets the options saved in the database or their default values
    |---------------------------------------------------------------------------------------------------
    */
    protected function set_options( $set_options = array(), $defaults = array() ){
        $important_options = array(
            TGR_PREFIX . 'id' => $this->id,
        );

        //Set prefix
        foreach( $defaults as $key => $val ){
            $option_name = $this->get_option_name( $key );
            unset( $defaults[$key] );
            $defaults[$option_name] = $val;
        }
        foreach( $set_options as $key => $val ){
            $option_name = $this->get_option_name( $key );
            unset( $set_options[$key] );
            $set_options[$option_name] = $val;
        }

        $options = wp_parse_args( $this->set_general_options(), $defaults );
        $options = wp_parse_args( $this->set_popup_options(), $options );
        $options = wp_parse_args( $this->set_inline_options(), $options );
        $options = wp_parse_args( $this->set_slide_options(), $options );
        $options = wp_parse_args( $this->set_displayer_options(), $options );
        $options = wp_parse_args( $this->set_customization_general_options(), $options );
        $options = wp_parse_args( $this->set_font_options(), $options );
        $options = wp_parse_args( $this->set_layout_options(), $options );
        $options = wp_parse_args( $this->set_grid_options(), $options );
        $options = wp_parse_args( $this->set_list_options(), $options );
        $options = wp_parse_args( $this->set_table_options(), $options );
        $options = wp_parse_args( $this->set_social_options(), $options );
        $options = wp_parse_args( $this->set_filter_options(), $options );
        $options = wp_parse_args( $this->set_skill_options(), $options );
        $options = wp_parse_args( $this->set_advanced_options(), $options );
        $options = wp_parse_args( $important_options, $options );
        $options = wp_parse_args( $set_options, $options );

        do_action( 'tgr_team_before_set_options', $options, $this );
        $this->options = $options;
        do_action( 'tgr_team_after_set_options', $this->options, $this );

        $this->options = apply_filters( 'tgr_team_options', $this->options, $this );

        return $this->options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Metadata options
    |---------------------------------------------------------------------------------------------------
    */
    public function metadata_options(){
        $options = array(
            'id' => $this->id,
            'showTitle' => $this->option( 'show-title' ) == 'on' || $this->option( 'show-title' ) == 'true',
            'layout' => array(
                'columns' => (int) $this->option( 'layout-columns' ),
            ),
            'className' => $this->get_team_classes(),
            'responsiveBy' => $this->option( 'responsive-by' ),
            'displayer' => array(
                'type' => $this->option('displayer')
            ),
        );
        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the options for the js plugin
    |---------------------------------------------------------------------------------------------------
    */
    public function js_options(){
        $options = array(
            'id' => $this->id,
            'title' => $this->title,
            'showTitle' => $this->option( 'show-title' ) == 'on' || $this->option( 'show-title' ) == 'true',
            'teamDescription' => $this->option( 'team-description' ),
            'refresh' => false,
            'loadUsingAjax' => $this->option( 'load-using-ajax' ) == 'on',
            'isDesign' => false,
            'isFilterDesign' => false,
            'members' => $this->get_members_ids(),
            'className' => $this->get_team_classes(),
            'responsiveBy' => $this->option( 'responsive-by' ),
            'bgColor' => $this->option('team-bg-color'),
            'font' => $this->get_font_options(),
            'layout' => $this->get_layout_options(),
            'social' => $this->get_social_options(),
            'skills' => $this->get_skill_options(),
            'filters' => $this->get_filter_options(),
            'member' => array(
                'titleTag' => $this->option( 'member-title-tag' ),
                'roleTag' => $this->option( 'member-subtitle-tag' ),
                'titleAfterRole' => $this->option( 'member-title-after-subtitle' ) == 'on',
            ),
            'displayer' => $this->get_displayer_options(),
        );
        return apply_filters( 'tgr_team_js_options', $options, $this );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the javascript variables with the plugin options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_js_options(){
        $return = '';
        $options = $this->js_options();
        $return .= "\n\n TGR_TEAMS[$this->id] = " . json_encode( $options ) . ";";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_general_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_general_options(){
        $options = array();
        $this->set_option_to( $options, 'layout-number', '1' );
        $this->set_option_to( $options, 'layout-columns', 4 );
        $this->set_option_to( $options, 'layout-columns-mobile', 1 );
        $this->set_option_to( $options, 'layout-columns-gap', 30 );
        $this->set_option_to( $options, 'select-members', array() );
        $this->set_option_to( $options, 'sort-members', '' );
        $this->set_option_to( $options, 'members', '' );

        //Additional Settings
        $this->set_option_to( $options, 'show-title', 'off' );
        $this->set_option_to( $options, 'team-description', '' );
        $this->set_option_to( $options, 'responsive-by', 'window' );
        $this->set_option_to( $options, 'team-class', '' );
        $this->set_option_to( $options, 'team-class-shortcode', '' );

        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_popup_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_popup_options(){
        $options = array();
        $this->set_option_to( $options, 'popup-width', 880 );
        $this->set_option_to( $options, 'popup-height', 540 );
        $this->set_option_to( $options, 'popup-width_unit', 'px' );
        $this->set_option_to( $options, 'popup-height_unit', 'px' );
        $this->set_option_to( $options, 'popup-border-radius', '0' );
        $this->set_option_to( $options, 'popup-next-item-effect', 'fade' );

        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_inline_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_inline_options(){
        $options = array();
        $this->set_option_to( $options, 'inline-height', 460 );
        $this->set_option_to( $options, 'inline-margin-top', 20 );

        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_slide_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_slide_options(){
        $options = array();
        $this->set_option_to( $options, 'slide-width', 480 );

        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_displayer_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_displayer_options(){
        $options = array();

        $this->set_option_to( $options, 'displayer', 'popup' );
        $this->set_option_to( $options, 'displayer-vertical-padding', 36 );
        $this->set_option_to( $options, 'displayer-horizontal-padding', 40 );
        $this->set_option_to( $options, 'displayer-target-link', '_blank' );

        //Viewer font size
        $this->set_option_to( $options, 'displayer-text-size', 16 );
        $this->set_option_to( $options, 'displayer-text-size-heading1', 19 );
        $this->set_option_to( $options, 'displayer-text-size-heading2', 17 );
        $this->set_option_to( $options, 'displayer-text-color', 'rgba(68, 68, 68, 1)' );
        $this->set_option_to( $options, 'displayer-link-color', 'rgba(0, 0, 0, 1)' );
        $this->set_option_to( $options, 'displayer-bg-color', 'rgba(255, 255, 255, 1)' );
        $this->set_option_to( $options, 'displayer-border-width', 1 );
        $this->set_option_to( $options, 'displayer-border-color', 'rgba(0, 0, 0, 0.3)' );
        $this->set_option_to( $options, 'displayer-triangle-color', 'rgba(0, 0, 0, 0.6)' );
        $this->set_option_to( $options, 'displayer-constrols-color', 'rgba(102, 102, 102, 1)' );

        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_displayer_options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_displayer_options(){
        return array(
            'type' => $this->option( 'displayer' ),
            'autoScroll' => true,
            'textSize' => (int) $this->option( 'displayer-text-size' ),
            'textSizeHeading1' => (int) $this->option( 'displayer-text-size-heading1' ),
            'textSizeHeading2' => (int) $this->option( 'displayer-text-size-heading2' ),
            'textColor' => $this->option( 'displayer-text-color' ),
            'linkColor' => $this->option( 'displayer-link-color' ),
            'bgColor' => $this->option( 'displayer-bg-color' ),
            'borderWidth' => (int) $this->option( 'displayer-border-width' ),
            'borderColor' => $this->option( 'displayer-border-color' ),
            'triangleColor' => $this->option( 'displayer-triangle-color' ),
            'controlsColor' => $this->option( 'displayer-constrols-color' ),
            'verticalPadding' => $this->option( 'displayer-vertical-padding' ),
            'horizontalPadding' => $this->option( 'displayer-horizontal-padding' ),
            'targetLink' => $this->option( 'displayer-target-link' ),

            'inline' => array(
                'height' => (int) $this->option( 'inline-height' ),
                'marginTop' => (int) $this->option( 'inline-margin-top' ),
            ),
            'slide' => array(
                'width' => (int) $this->option( 'slide-width' ),
            ),

            'popup' => array(
                'width' => (int) $this->option( 'popup-width' ),
                'height' => (int) $this->option( 'popup-height' ),
                'borderRadius' => (int) $this->option( 'popup-border-radius' ),
                'nextItemEffect' => $this->option( 'popup-next-item-effect' ),//slide,fade
                'disablePageScroll' => true,
                'open' => array(
                    'delay' => 0,
                    'duration' => 800,
                    'animation' => $this->get_animation_effect_open_popup(),
                ),
                'close' => array(
                    'delay' => 0,
                    'duration' => 700,
                    'animation' => $this->get_animation_effect_close_popup( $this->get_animation_effect_open_popup() ),
                    'icon' => 'default',
                ),
                'overlay' => array(
                    'show' => true,
                    'durationIn' => 300,
                    'durationOut' => 250,
                ),
                'arrows' => array(
                    'icon' => 'default',
                )
            )
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_customization_general_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_customization_general_options(){
        $options = array();
        $this->set_option_to( $options, 'team-bg-color', 'rgba(255, 255, 255, 0)' );
        $this->set_option_to( $options, 'layout-truncate-description', -1 );
        $this->set_option_to( $options, 'member-title-after-subtitle', 'off' );
        $this->set_option_to( $options, 'show-member-headings', 'on' );
        $this->set_option_to( $options, 'grid-social-icons-visible', 'off' );
        $this->set_option_to( $options, 'grid-description-visible', 'off' );
        $this->set_option_to( $options, 'grid-description-visible-hover', 'off' );

        $this->set_option_to( $options, 'layout-hide-image', 'off' );
        $this->set_option_to( $options, 'layout-image-zoom-scale', 1.2 );
        $this->set_option_to( $options, 'layout-image-zoom-duration', 0.5 );
        $this->set_option_to( $options, 'layout-image-zoom-effect', 'ease-out' );

        $this->set_option_to( $options, 'layout-equal-heights', 'on' );

        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_font_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_font_options(){
        $options = array();

        $this->set_option_to( $options, 'font-family-heading1', 'Roboto' );
        $this->set_option_to( $options, 'font-family-heading2', 'Lato' );
        $this->set_option_to( $options, 'font-family-content', 'Open Sans' );
        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_font_options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_font_options(){
        return array(
            'heading1' => $this->option( 'font-family-heading1' ),
            'heading2' => $this->option( 'font-family-heading2' ),
            'content' => $this->option( 'font-family-content' ),
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_layout_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_layout_options(){
        $options = array();

        //Layout font size
        $this->set_option_to( $options, 'layout-text-size', 16 );
        $this->set_option_to( $options, 'layout-text-color', 'rgba(255, 255, 255, 1)' );
        $this->set_option_to( $options, 'layout-text-color2', 'rgba(255, 255, 255, 1)' );
        $this->set_option_to( $options, 'layout-link-color', 'rgba(0, 0, 0, 1)' );

        //Layout Bg
        $this->set_option_to( $options, 'layout-bg-color', 'rgba(0, 0, 0, 0.8)' );
        $this->set_option_to( $options, 'layout-bg-gradient-enabled', 'off' );
        $this->set_option_to( $options, 'layout-bg-gradient-color', 'rgba(0, 0, 0, 1)' );
        $this->set_option_to( $options, 'layout-bg-gradient-angle', 180 );
        $this->set_option_to( $options, 'layout-bg-color2', 'rgba(0, 0, 0, 0.8)' );
        $this->set_option_to( $options, 'layout-bg-gradient-enabled2', 'off' );
        $this->set_option_to( $options, 'layout-bg-gradient-color2', 'rgba(0, 0, 0, 1)' );
        $this->set_option_to( $options, 'layout-bg-gradient-angle2', 180 );

        //Layout border
        $this->set_option_to( $options, 'layout-border-width', 0 );
        $this->set_option_to( $options, 'layout-border-color', 'rgba(0, 0, 0, 0.3)' );
        $this->set_option_to( $options, 'layout-border-different-hover', 'off' );
        $this->set_option_to( $options, 'layout-border-width2', 0 );
        $this->set_option_to( $options, 'layout-border-color2', 'rgba(0, 0, 0, 0.3)' );
        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_layout_options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_layout_options(){
        return array(
            'number' => $this->option( 'layout-number' ),
            'type' => $this->get_layout_type( $this->option( 'layout-number' ) ),
            'columns' => (int) $this->option( 'layout-columns' ),
            'columnsMobile' => (int) $this->option( 'layout-columns-mobile' ),
            'columnsGap' => (int) $this->option( 'layout-columns-gap' ),
            'truncateDescription' => (int) $this->option( 'layout-truncate-description' ),

            'hideImage' => $this->option( 'layout-hide-image' ) == 'on',
            'imageZoomScale' => (float) $this->option( 'layout-image-zoom-scale' ),
            'imageZoomDuration' => (float) $this->option( 'layout-image-zoom-duration' ),
            'imageZoomEffect' => $this->option( 'layout-image-zoom-effect' ),

            'textSize' => (int) $this->option( 'layout-text-size' ),
            'textSizeHeading1' => (int) $this->option( 'layout-text-size-heading1' ),
            'textSizeHeading2' => (int) $this->option( 'layout-text-size-heading2' ),

            'textColor' => $this->option( 'layout-text-color' ),
            'textColor2' => $this->option( 'layout-text-color2' ),
            'linkColor' => $this->option( 'layout-link-color' ),

            'bgColor' => $this->option( 'layout-bg-color' ),
            'bgGradientEnabled' => $this->option( 'layout-bg-gradient-enabled' ) == 'on',
            'bgGradientColor' => $this->option( 'layout-bg-gradient-color' ),
            'bgGradientAngle' => CSS::number( $this->option( 'layout-bg-gradient-angle' ), 'deg' ),
            'bgColor2' => $this->option( 'layout-bg-color2' ),
            'bgGradientEnabled2' => $this->option( 'layout-bg-gradient-enabled2' ) == 'on',
            'bgGradientColor2' => $this->option( 'layout-bg-gradient-color2' ),
            'bgGradientAngle2' => CSS::number( $this->option( 'layout-bg-gradient-angle2' ), 'deg' ),

            'borderWidth' => (int) $this->option( 'layout-border-width' ),
            'borderColor' => $this->option( 'layout-border-color' ),
            'borderDifferentHover' => $this->option( 'layout-border-different-hover' ) == 'on',
            'borderWidth2' => (int) $this->option( 'layout-border-width2' ),
            'borderColor2' => $this->option( 'layout-border-color2' ),

            'equalHeights' => $this->option( 'layout-equal-heights' ) == 'on',

            'grid' => $this->get_grid_options(),
            'list' => $this->get_list_options(),
            'table' => $this->get_table_options()
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_grid_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_grid_options(){
        $options = array();
        $this->set_option_to( $options, 'member-title-tag', 'h3' );
        $this->set_option_to( $options, 'member-subtitle-tag', 'h4' );
        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_grid_options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_grid_options(){
        return array(
            'headings' => array(
                'visible' => $this->option( 'show-member-headings' ) == 'on',
            ),
            'socialIcons' => array(
                'visible' => $this->option( 'grid-social-icons-visible' ) == 'on',
            ),
            'description' => array(
                'visible' => $this->option( 'grid-description-visible' ) == 'on',
                'visibleOnHover' => $this->option( 'grid-description-visible-hover' ) == 'on',
            ),
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_list_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_list_options(){
        $options = array();
        $this->set_option_to( $options, 'list-is-right', 'off' );
        $this->set_option_to( $options, 'list-border-bottom-only', 'on' );

        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_list_options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_list_options(){
        return array(
            'isRight' => $this->option( 'list-is-right' ) == 'on',
            'borderBottomOnly' => $this->option( 'list-border-bottom-only' ) == 'on',
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_table_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_table_options(){
        $options = array();
        $this->set_option_to( $options, 'table-bordered', 'off' );
        $this->set_option_to( $options, 'table-striped', 'off' );
        $this->set_option_to( $options, 'table-hover', 'off' );
        $this->set_option_to( $options, 'table-striped-color', 'rgba(0, 0, 0, 0.03)' );
        $this->set_option_to( $options, 'table-hover-color', 'rgba(0, 0, 0, 0.04)' );
        $this->set_option_to( $options, 'table-columns', array( 'image', 'name', 'role', 'description', 'social' ) );

        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_table_options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_table_options(){
        return array(
            'bordered' => $this->option( 'table-bordered' ) == 'on',
            'striped' => $this->option( 'table-striped' ) == 'on',
            'hover' => $this->option( 'table-hover' ) == 'on',
            'stripedColor' => $this->option( 'table-striped-color' ),
            'hoverColor' => $this->option( 'table-hover-color' ),
            'columns' => $this->option( 'table-columns' ),
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_social_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_social_options(){
        $options = array();
        $this->set_option_to( $options, 'social-icons-color', 'rgba(51, 51, 51, 1)' );
        $this->set_option_to( $options, 'social-icons-color-hover', 'rgba(0, 0, 0, 1)' );
        $this->set_option_to( $options, 'social-icons-border-radius', 50 );
        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_social_options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_social_options(){
        return array(
            'iconsColor' => $this->option( 'social-icons-color' ),
            'iconsColorHover' => $this->option( 'social-icons-color-hover' ),
            'iconsBorderRadius' => $this->option( 'social-icons-border-radius' ),
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_filter_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_filter_options(){
        $options = array();
        $this->set_option_to( $options, 'filters-show', 'on' );
        $this->set_option_to( $options, 'filters-selected', array() );
        $this->set_option_to( $options, 'filters-sort', 'none' );
        $this->set_option_to( $options, 'filters-hide-all', 'off' );
        $this->set_option_to( $options, 'filters-multiple-choice', 'off' );
        $this->set_option_to( $options, 'filters-show-filter-count', 'off' );
        $this->set_option_to( $options, 'filters-dropdown', 'off' );

        $this->set_option_to( $options, 'filters-gap', 4 );
        $this->set_option_to( $options, 'filters-font-family', 'Roboto Condensed' );
        $this->set_option_to( $options, 'filters-uppercase-text', 'on' );
        $this->set_option_to( $options, 'filters-text-size', 16 );
        $this->set_option_to( $options, 'filters-text-color', 'rgba(76, 76, 76, 1)' );
        $this->set_option_to( $options, 'filters-border-width', 0 );
        $this->set_option_to( $options, 'filters-border-color', 'rgba(0, 0, 0, 0)' );
        $this->set_option_to( $options, 'filters-border-radius', 0 );
        $this->set_option_to( $options, 'filters-padding', '12px 24px' );
        $this->set_option_to( $options, 'filters-bg-color', 'rgba(255, 255, 255, 1)' );
        $this->set_option_to( $options, 'filters-bg-gradient-enabled', 'off' );
        $this->set_option_to( $options, 'filters-bg-gradient-color', 'rgba(0, 0, 0, 1)' );
        $this->set_option_to( $options, 'filters-bg-gradient-angle', 135 );

        $this->set_option_to( $options, 'filters-text-color-active', 'rgba(0, 0, 0, 1)' );
        $this->set_option_to( $options, 'filters-border-width-active', 3 );
        $this->set_option_to( $options, 'filters-border-color-active', 'rgba(252,1,41,1)' );
        $this->set_option_to( $options, 'filters-border-bottom-only-active', 'on' );
        $this->set_option_to( $options, 'filters-bg-color-active', 'rgba(255, 255, 255, 0)' );
        $this->set_option_to( $options, 'filters-bg-gradient-enabled-active', 'off' );
        $this->set_option_to( $options, 'filters-bg-gradient-color-active', 'rgba(0, 0, 0, 1)' );
        $this->set_option_to( $options, 'filters-bg-gradient-angle-active', 135 );

        $this->set_option_to( $options, 'filters-text-color-hover', 'rgba(0, 0, 0, 1)' );
        $this->set_option_to( $options, 'filters-border-color-hover', 'rgba(252,1,41,1)' );
        $this->set_option_to( $options, 'filters-bg-color-hover', 'rgba(255, 255, 255, 0)' );
        $this->set_option_to( $options, 'filters-bg-gradient-enabled-hover', 'off' );
        $this->set_option_to( $options, 'filters-bg-gradient-color-hover', 'rgba(0, 0, 0, 1)' );
        $this->set_option_to( $options, 'filters-bg-gradient-angle-hover', 135 );


        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_filter_options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_filter_options(){
        return array(
            'show' => $this->option( 'filters-show' ) == 'on',
            'selected' => $this->option( 'filters-selected' ),
            'sort' => $this->option( 'filters-sort' ),
            'hideAll' => $this->option( 'filters-hide-all' ) == 'on',
            'multipleChoice' => $this->option( 'filters-multiple-choice' ) == 'on',
            'showFilterCount' => $this->option( 'filters-show-filter-count' ) == 'on',
            'inEffect' => $this->is_layout_table() ? 'tgr-fadeIn' : 'tgr-zoomIn',
            'outEffect' => $this->is_layout_table() ? 'tgr-fadeOut' : 'tgr-zoomOut',
            'duration' => 630,
            'dropdown' => $this->option( 'filters-dropdown' ) == 'on',

            'gap' => (int) $this->option( 'filters-gap' ),
            'textColor' => $this->option( 'filters-text-color' ),
            'fontFamily' => $this->option( 'filters-font-family' ),
            'uppercaseText' => $this->option( 'filters-uppercase-text' ) == 'on',
            'textSize' => (int) $this->option( 'filters-text-size' ),
            'borderWidth' => (int) $this->option( 'filters-border-width' ),
            'borderColor' => $this->option( 'filters-border-color' ),
            'borderRadius' => (int) $this->option( 'filters-border-radius' ),
            'padding' => $this->option( 'filters-padding' ),
            'bgColor' => $this->option( 'filters-bg-color' ),
            'bgGradientEnabled' => $this->option( 'filters-bg-gradient-enabled' ) == 'on',
            'bgGradientColor' => $this->option( 'filters-bg-gradient-color' ),
            'bgGradientAngle' => CSS::number( $this->option( 'filters-bg-gradient-angle' ), 'deg' ),

            'textColorActive' => $this->option( 'filters-text-color-active' ),
            'borderWidthActive' => (int) $this->option( 'filters-border-width-active' ),
            'borderColorActive' => $this->option( 'filters-border-color-active' ),
            'borderBottomOnlyActive' => $this->option( 'filters-border-bottom-only-active' ) == 'on',
            'bgColorActive' => $this->option( 'filters-bg-color-active' ),
            'bgGradientEnabledActive' => $this->option( 'filters-bg-gradient-enabled-active' ) == 'on',
            'bgGradientColorActive' => $this->option( 'filters-bg-gradient-color-active' ),
            'bgGradientAngleActive' => CSS::number( $this->option( 'filters-bg-gradient-angle-active' ), 'deg' ),

            'textColorHover' => $this->option( 'filters-text-color-hover' ),
            'borderColorHover' => $this->option( 'filters-border-color-hover' ),
            'bgColorHover' => $this->option( 'filters-bg-color-hover' ),
            'bgGradientEnabledHover' => $this->option( 'filters-bg-gradient-enabled-hover' ) == 'on',
            'bgGradientColorHover' => $this->option( 'filters-bg-gradient-color-hover' ),
            'bgGradientAngleHover' => CSS::number( $this->option( 'filters-bg-gradient-angle-hover' ), 'deg' ),
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_skill_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_skill_options(){
        $options = array();
        $this->set_option_to( $options, 'skills-bg-color', 'rgba(0, 0, 0, 0.2)' );
        $this->set_option_to( $options, 'skills-bg-percent', 'rgba(0, 0, 0, 0.7)' );
        $this->set_option_to( $options, 'skills-show-percent', 'on' );
        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_skill_options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_skill_options(){
        return array(
            'bgColor' => $this->option( 'skills-bg-color' ),
            'bgPercent' => $this->option( 'skills-bg-percent' ),
            'showPercent' => $this->option( 'skills-show-percent' ) == 'on',
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_team_classes
    |---------------------------------------------------------------------------------------------------
    */
    public function get_team_classes(){
        if( $this->option( 'team-class-shortcode' ) != '' ){
            return $this->option( 'team-class' ) . ' ' . $this->option( 'team-class-shortcode' );
        }
        return $this->option( 'team-class' );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | set_advanced_options
    |---------------------------------------------------------------------------------------------------
    */
    public function set_advanced_options(){
        $options = array();
        $this->set_option_to( $options, 'custom-css', '' );
        $this->set_option_to( $options, 'custom-javascript', '' );
        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Lista de miembros
    |---------------------------------------------------------------------------------------------------
    */
    public function get_members_ids( $published_only = true ){
        //Comma separated ordered list
        $members = $this->option( 'members' );
        if( empty( $members ) ){
            return array();
        }
        $member_ids = array_map( 'trim', explode( ',', $members ) );

        if( $this->option( 'sort-members' ) == 'random' ){
            shuffle( $member_ids );
            update_post_meta( $this->id, TGR_PREFIX.'members', implode( ',', $member_ids ) );
        }
        //Get only published members
        $published_ids = array();
        foreach( $member_ids as $member_id ){
            if( $this->plugin->is_published_member( $member_id ) ){
                $published_ids[] = $member_id;
            }
        }
        return $published_only ? $published_ids : $member_ids;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_selected_members_ids
    |---------------------------------------------------------------------------------------------------
    */
    public function get_selected_members_ids(){
        $select_members = (array) $this->option( 'select-members', array() );
        return array_filter( $select_members );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if it is a specific layout
    |---------------------------------------------------------------------------------------------------
    */
    public function is_layout( $layout_number ){
        if( is_array( $layout_number ) ){
            return in_array( $this->option( 'layout-number' ), $layout_number );
        }
        return $this->option( 'layout-number' ) == $layout_number;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if it is list layout
    |---------------------------------------------------------------------------------------------------
    */
    public function is_layout_list(){
        return $this->option( 'layout-number' ) == '16';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if it is table layout
    |---------------------------------------------------------------------------------------------------
    */
    public function is_layout_table(){
        return $this->option( 'layout-number' ) == '17';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Verifica si es flip card layout
    |---------------------------------------------------------------------------------------------------
    */
    public function is_layout_flip_card(){
        if( $this->option( 'layout-number' ) == '13' ){
            return true;
        }
        $description_visible = $this->option( 'grid-description-visible' ) == 'on';
        if( in_array( $this->option( 'layout-number' ), array( '10', '12' ) ) && ! $description_visible ){
            return true;
        }
        return false;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get layout type
    |---------------------------------------------------------------------------------------------------
    */
    public function get_layout_type( $number ){
        if( isset( self::$layouts[$number] ) ){
            return self::$layouts[$number];
        }
        return 'grid';
    }

    /*
   |---------------------------------------------------------------------------------------------------
   | get_animation_effect_open_popup
   |---------------------------------------------------------------------------------------------------
   */
    public function get_animation_effect_open_popup(){
        if( $this->is_displayer_slide_in( 'slide-in-left' ) ){
            return 'tgr-slideInLeft';
        } else if( $this->is_displayer_slide_in( 'slide-in-right' ) ){
            return 'tgr-slideInRight';
        }
        return 'tgr-zoomIn';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_animation_effect_close_popup
    |---------------------------------------------------------------------------------------------------
    */
    public function get_animation_effect_close_popup( $open_effect ){
        $default_effect = 'tgr-zoomOut';
        if( $this->is_displayer_slide_in( 'slide-in-left' ) ){
            $default_effect = 'tgr-slideOutLeft';
        } else if( $this->is_displayer_slide_in( 'slide-in-right' ) ){
            $default_effect = 'tgr-slideOutRight';
        }
        $all_effects = Assets::animation_effects();
        return isset( $all_effects[$open_effect] ) ? $all_effects[$open_effect] : $default_effect;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | is_displayer_none
    |---------------------------------------------------------------------------------------------------
    */
    public function is_displayer_none(){
        return $this->option( 'displayer' ) == 'none';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | is_displayer_popup
    |---------------------------------------------------------------------------------------------------
    */
    public function is_displayer_popup(){
        return $this->option( 'displayer' ) == 'popup';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | is_displayer_slide_in
    |---------------------------------------------------------------------------------------------------
    */
    public function is_displayer_slide_in( $position = null ){
        if( $position ){
            return $this->option( 'displayer' ) == $position;
        }
        return $this->option( 'displayer' ) == 'slide-in-left' || $this->option( 'displayer' ) == 'slide-in-right';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | is_displayer_inline
    |---------------------------------------------------------------------------------------------------
    */
    public function is_displayer_inline(){
        return $this->option( 'displayer' ) == 'inline';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_grid_background
    |---------------------------------------------------------------------------------------------------
    */
    public function get_grid_background( $status = 1, $gradient = false ){
        $lo = $this->get_layout_options();
        if( $status == 2 ){
            if( $gradient ){
                return "linear-gradient({$lo['bgGradientAngle2']}, {$lo['bgColor2']}, {$lo['bgGradientColor2']} )";
            } else{
                return $lo['bgColor2'];
            }
        } else{
            if( $gradient ){
                return "linear-gradient({$lo['bgGradientAngle']}, {$lo['bgColor']}, {$lo['bgGradientColor']} )";
            } else{
                return $lo['bgColor'];
            }
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_fonts
    |---------------------------------------------------------------------------------------------------
    */
    public function get_fonts(){
        $font = $this->get_font_options();
        $filters = $this->get_filter_options();
        $this->fonts[] = $font['content'];
        $this->fonts[] = $font['heading1'];
        $this->fonts[] = $font['heading2'];
        $this->fonts[] = $filters['fontFamily'];

        $fonts = array_values( array_unique( array_filter( $this->fonts ) ) );
        return $fonts;
    }

}