<?php namespace TeamGrid\Includes;

use Xbox\Includes\CSS;

class Team extends TeamOptions {
    protected static $instance = null;
    public $plugin;

    public $layout_number = 1;
    public $fonts = array();


    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    public function __construct( $options = array(), $defaults = array() ){
        $this->plugin = Functions::get_plugin_instance();

        parent::__construct( $options, $defaults );

        $this->init();
        $this->hooks();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Init
    |---------------------------------------------------------------------------------------------------
    */
    private function init(){
        $this->layout_number = $this->option( 'layout-number' );
        $this->layout_type = $this->get_layout_type( $this->layout_number );
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Renderizar html
    |---------------------------------------------------------------------------------------------------
    */
    public function build(){
        $return = '';
        $options = json_encode( $this->metadata_options() );
        $return .= "<div class='tgr-team' data-id='" . esc_attr( $this->id ) . "' data-options='$options'>";
        $return .= "<div class='" . esc_attr( $this->get_box_wrapper_class() ) . "'>";
        $return .= "<div class='tgr-box'>";
        $return .= "<div class='" . esc_attr( $this->get_box_inner_class() ) . "'>";

        if( $this->option('load-using-ajax') == 'on'){
            $return .= "Loading team using Ajax...";
        } else{
            $return .= $this->build_items();
        }

        $return .= "</div>";//.tgr-box-inner
        $return .= "</div>";//.tgr-box
        $return .= "</div>";//.tgr-box-wrapper
        $return .= "</div>";//.tgr-team

        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build Members
    |---------------------------------------------------------------------------------------------------
    */
    public function build_items(){
        $return = "";
        if( $this->is_layout_table() ){
            $return .= $this->build_items_table();
        } else{
            $return .= $this->build_items_grid();
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_box_wrapper_class
    |---------------------------------------------------------------------------------------------------
    */
    public function get_box_wrapper_class(){
        $class_wrap = array( 'tgr-box-wrapper' );
        $class_wrap[] = 'tgr-anim-zoom-images';
        $class_wrap[] = "tgr-displayer-{$this->option('displayer')}";
        $class_wrap[] = "tgr-layout-{$this->option('layout-number')}";
        $class_wrap[] = "tgr-layout-type-{$this->layout_type}";
        if( $this->option( 'layout-hide-image' ) == 'on' ){
            $class_wrap[] = "tgr-hide-image";
        }
        $class_wrap = trim( implode( ' ', $class_wrap ) );
        return $class_wrap;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_box_inner_class
    |---------------------------------------------------------------------------------------------------
    */
    public function get_box_inner_class(){
        $class_box_inner = array( 'tgr-box-inner' );
        if( ! $this->is_layout_table() && ! $this->is_layout_list() ){
            $class_box_inner[] = 'tgr-grid';
            $class_box_inner[] = 'tgr-grid-' . $this->option( 'layout-columns' ) . '-col';
            if( $this->option( 'layout-columns-mobile' ) == 2 ){
                $class_box_inner[] = 'tgr-mobile-2-columns';
            }
        }
        $class_box_inner = trim( implode( ' ', $class_box_inner ) );
        return $class_box_inner;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_items_grid
    |---------------------------------------------------------------------------------------------------
    */
    public function build_items_grid(){
        $return = '';
        $index = 0;
        foreach( $this->get_members_ids() as $member_id ){
            $member = Members::get( $member_id );
            if( $member ){
                $member->set_team( $this );
                $return .= $member->render( $index );
                $index++;
            }
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build the items for the table layout
    |---------------------------------------------------------------------------------------------------
    */
    public function build_items_table(){
        $return = '';
        $table = $this->get_table_options();

        $return .= "<div class='" . esc_attr( $this->get_table_class() ) . "'>";

        $return .= "<div class='tgr-tr tgr-thead'>";
        if( in_array( 'image', $table['columns'] ) ){
            $return .= "<div class='tgr-th'>" . esc_html( $this->plugin->settings->option( 'table-text-image' ) ) . "</div>";
        }
        if( in_array( 'name', $table['columns'] ) ){
            $return .= "<div class='tgr-th'>" . esc_html( $this->plugin->settings->option( 'table-text-name' ) ) . "</div>";
        }
        if( in_array( 'role', $table['columns'] ) ){
            $return .= "<div class='tgr-th'>" . esc_html( $this->plugin->settings->option( 'table-text-role' ) ) . "</div>";
        }
        if( in_array( 'description', $table['columns'] ) ){
            $return .= "<div class='tgr-th'>" . esc_html( $this->plugin->settings->option( 'table-text-description' ) ) . "</div>";
        }
        if( in_array( 'social', $table['columns'] ) ){
            $return .= "<div class='tgr-th'>" . esc_html( $this->plugin->settings->option( 'table-text-social-icons' ) ) . "</div>";
        }
        if( in_array( 'fields', $table['columns'] ) ){
            $return .= "<div class='tgr-th'>" . esc_html( $this->plugin->settings->option( 'table-text-fields' ) ) . "</div>";
        }
        if( in_array( 'skills', $table['columns'] ) ){
            $return .= "<div class='tgr-th'>" . esc_html( $this->plugin->settings->option( 'table-text-skills' ) ) . "</div>";
        }
        $return .= "</div>";//tgr-thead

        $index = 0;
        foreach( $this->get_members_ids() as $member_id ){
            $member = Members::get( $member_id );
            if( $member ){
                $member->set_team( $this );
                $return .= $member->render( $index );
                $index++;
            }
        }
        $return .= "</div>";//tgr-table
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_table_class
    |---------------------------------------------------------------------------------------------------
    */
    public function get_table_class(){
        $table_class = array( 'tgr-table' );
        $bordered = $this->option( 'table-bordered' ) == 'on';
        $striped = $this->option( 'table-striped' ) == 'on';
        $hover = $this->option( 'table-hover' ) == 'on';
        if( $bordered ){
            $table_class[] = 'tgr-table-bordered';
        }
        if( $striped ){
            $table_class[] = 'tgr-table-striped';
        }
        if( $hover ){
            $table_class[] = 'tgr-table-hover';
        }
        return implode( ' ', $table_class );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get link google fonts
    |---------------------------------------------------------------------------------------------------
    */
    public function get_link_google_fonts(){
        if( $this->plugin->settings->option( 'load-google-fonts' ) == "off" ){
            return "";
        }
        $href = Functions::url_google_fonts( $this->get_fonts(), array_values( Assets::local_fonts() ) );
        if( $href ){
            return '<link class="tgr-google-fonts" href="' . esc_url( $href ) . '" rel="stylesheet" type="text/css">';
        }
        return "";
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Build the css for the team
    |---------------------------------------------------------------------------------------------------
    */
    public function build_style(){
        $style = '<style class="tgr-style-backend" data-id="' . esc_attr( $this->id ) . '">';

        $style .= $this->get_font_styles();
        $style .= $this->get_border_styles();
        $style .= $this->get_grid_styles();
        $style .= $this->get_list_styles();
        $style .= $this->get_table_styles();
        $style .= '</style>';
        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | build_custom_style
    |---------------------------------------------------------------------------------------------------
    */
    public function build_custom_style(){
        $style = '<style class="tgr-custom-css">';
        $style .= str_replace( '[id]', $this->id, $this->option( 'custom-css' ) );
        $style .= '</style>';
        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build custom javascript
    |---------------------------------------------------------------------------------------------------
    */
    public function build_custom_script(){
        $return = "";
        $return .= "<script>";
        $return .= "\n//Custom javascript\n";
        /*
         * The setting below is not sanitized. In order to be able to take advantage of this,
		 * a user would have to gain access to the database,
		 * in which case this is the least of your worries.
         */
        $return .= $this->option( 'custom-javascript' );
        $return .= '</script>';
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_font_styles
    |---------------------------------------------------------------------------------------------------
    */
    public function get_font_styles(){
        $style = '';
        $font = $this->get_font_options();
        $layout = $this->get_layout_options();

        //Font family
        $css = new CSS( ".tgr-team-$this->id .tgr-item-desc, .tgr-team-$this->id .tgr-member-content, .tgr-team-$this->id .tgr-member-fields" );
        $css->prop( 'font-family', $font['content'] . ', sans-serif' );
        $style .= $css->build_css();

        $css = new CSS( ".tgr-team-$this->id .tgr-item-title, .tgr-team-$this->id .tgr-member-title, .tgr-team-$this->id .tgr-box-wrapper .tgr-member-content h1" );
        $css->prop( 'font-family', $font['heading1'] . ', sans-serif' );
        $style .= $css->build_css();

        $css = new CSS( ".tgr-team-$this->id .tgr-item-subtitle, .tgr-team-$this->id .tgr-member-role, .tgr-team-$this->id .tgr-box-wrapper .tgr-member-content h2, .tgr-team-$this->id .tgr-box-wrapper .tgr-member-content h3, .tgr-team-$this->id .tgr-box-wrapper .tgr-member-content h4, .tgr-team-$this->id .tgr-member-social h4, .tgr-team-$this->id  .tgr-member-fields h4, .tgr-team-$this->id  .tgr-member-skills h4" );
        $css->prop( 'font-family', $font['heading2'] . ', sans-serif' );
        $style .= $css->build_css();

        //Text color
        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-figcaption, .tgr-team-$this->id .tgr-layout-type-grid .tgr-item-image, .tgr-team-$this->id .tgr-layout-type-table .tgr-table, .tgr-team-$this->id .tgr-layout-type-list .tgr-member" );
        $css->prop( 'color', $layout['textColor'] );
        $style .= $css->build_css();

        if( $this->is_layout( array( 1 ) ) ){
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-image" );
            $css->prop( 'color', $layout['textColor2'] );
            $style .= $css->build_css();
        } else if( $this->is_layout( array( 8 ) ) ){// 7
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner:hover .tgr-item-image, .tgr-team-$this->id .tgr-layout-type-grid .tgr-grid-item.tgr-active .tgr-item-inner .tgr-item-image" );
            $css->prop( 'color', $layout['textColor2'] );
            $style .= $css->build_css();
        } else if( $this->is_layout( array( 10, 12 ) ) ){// 9, 11
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-flip-back .tgr-figcaption, .tgr-team-$this->id .tgr-layout-type-grid .tgr-grid-item.tgr-active .tgr-item-inner .tgr-figcaption" );
            $css->prop( 'color', $layout['textColor2'] );
            $style .= $css->build_css();
        } else if( $this->is_layout( array( 7 ) ) ){// 6, 9, 11
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner:hover .tgr-figcaption, .tgr-team-$this->id .tgr-layout-type-grid .tgr-grid-item.tgr-active .tgr-item-inner .tgr-figcaption, .tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner:hover .tgr-item-image, .tgr-team-$this->id .tgr-layout-type-grid .tgr-grid-item.tgr-active .tgr-item-inner .tgr-item-image" );
            $css->prop( 'color', $layout['textColor2'] );
            $style .= $css->build_css();
        }

        //Links color
        $css = new CSS( ".tgr-team-$this->id .tgr-member a:not(.tgr-item-link)" );
        $css->prop( 'color', $layout['linkColor'] );
        $style .= $css->build_css();

        //Text size
        if( $this->is_layout_table() ){
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-table .tgr-table" );
            $css->prop( 'font-size', CSS::number( $layout['textSize'], 'px' ) );
            $style .= $css->build_css();

            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-table .tgr-table .tgr-thead" );
            $css->prop( 'font-size', CSS::number( $layout['textSizeHeading1'], 'px' ) );
            $style .= $css->build_css();
        } else{
            $css = new CSS( ".tgr-team-$this->id .tgr-figure, .tgr-team-$this->id .tgr-item-inner .tgr-member-content, .tgr-team-$this->id .tgr-item-inner .tgr-member-fields, .tgr-team-$this->id .tgr-item-inner .tgr-member-skills" );
            $css->prop( 'font-size', CSS::number( $layout['textSize'], 'px' ) );
            $style .= $css->build_css();

            $css = new CSS( ".tgr-team-$this->id .tgr-item-title, .tgr-team-$this->id .tgr-item-inner .tgr-member-title" );
            $css->prop( 'font-size', CSS::number( $layout['textSizeHeading1'], 'px' ) );
            $style .= $css->build_css();

            $css = new CSS( ".tgr-team-$this->id .tgr-item-subtitle, .tgr-team-$this->id .tgr-item-inner .tgr-member-role, .tgr-team-$this->id .tgr-member-social h4, .tgr-team-$this->id  .tgr-member-fields h4, .tgr-team-$this->id  .tgr-member-skills h4" );
            $css->prop( 'font-size', CSS::number( $layout['textSizeHeading2'], 'px' ) );
            $style .= $css->build_css();
        }

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_border_styles
    |---------------------------------------------------------------------------------------------------
    */
    public function get_border_styles(){
        $style = '';
        $layout = $this->get_layout_options();

        if( $this->is_layout( array( 9, 11 ) ) ){//8, 10
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner" );
            $css->prop( 'border-width', CSS::number( $layout['borderWidth'], 'px' ) );
            $css->prop( 'border-color', $layout['borderColor'] );
            $style .= $css->build_css();
            if( $layout['borderDifferentHover'] ){
                $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner:hover" );
                $css->prop( 'border-width', CSS::number( $layout['borderWidth2'], 'px' ) );
                $css->prop( 'border-color', $layout['borderColor2'] );
                $style .= $css->build_css();
            }
        } else if( $this->is_layout( array( 13 ) ) ){//12
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-image, .tgr-team-$this->id .tgr-layout-type-grid .tgr-flip-back" );
            $css->prop( 'border-width', CSS::number( $layout['borderWidth'], 'px' ) );
            $css->prop( 'border-color', $layout['borderColor'] );
            $style .= $css->build_css();
            if( $layout['borderDifferentHover'] ){
                $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-flip-back" );
                $css->prop( 'border-width', CSS::number( $layout['borderWidth2'], 'px' ) );
                $css->prop( 'border-color', $layout['borderColor2'] );
                $style .= $css->build_css();
            }
        } else if( $this->is_layout( array( 14 ) ) ){//13
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner:before" );
            $css->prop( 'border-width', CSS::number( $layout['borderWidth'], 'px' ) );
            $css->prop( 'border-color', $layout['borderColor'] );
            $style .= $css->build_css();
            if( $layout['borderDifferentHover'] ){
                $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner:hover:before" );
                $css->prop( 'border-width', CSS::number( $layout['borderWidth2'], 'px' ) );
                $css->prop( 'border-color', $layout['borderColor2'] );
                $style .= $css->build_css();
            }
        } else if( $this->is_layout( array( 15 ) ) ){//14
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-image, .tgr-team-$this->id .tgr-layout-type-grid .tgr-figcaption" );
            $css->prop( 'border-width', CSS::number( $layout['borderWidth'], 'px' ) );
            $css->prop( 'border-color', $layout['borderColor'] );
            $style .= $css->build_css();
            if( $layout['borderDifferentHover'] ){
                $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-figcaption" );
                $css->prop( 'border-width', CSS::number( $layout['borderWidth2'], 'px' ) );
                $css->prop( 'border-color', $layout['borderColor2'] );
                $style .= $css->build_css();
            }
        } else{
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-figure" );
            $css->prop( 'border-width', CSS::number( $layout['borderWidth'], 'px' ) );
            $css->prop( 'border-color', $layout['borderColor'] );
            $style .= $css->build_css();
            if( $layout['borderDifferentHover'] ){
                $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner:hover .tgr-figure" );
                $css->prop( 'border-width', CSS::number( $layout['borderWidth2'], 'px' ) );
                $css->prop( 'border-color', $layout['borderColor2'] );
                $style .= $css->build_css();
            }
        }

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_grid_styles
    |---------------------------------------------------------------------------------------------------
    */
    public function get_grid_styles(){
        $style = '';
        $displayer = $this->get_displayer_options();

        //Inline arrow
        $css = new CSS( ".tgr-team-$this->id .tgr-displayer-inline .tgr-item-inner:after" );
        $css->prop( 'border-bottom-color', $displayer['bgColor'] );
        $style .= $css->build_css();

        if( $this->is_layout( array( 1, 2, 3, 4, 5, 6 ) ) ){//1, 2, 3, 4, 5
            $style .= $this->get_styles_to_grid_layouts();
        } else if( $this->is_layout( 7 ) ){//6
            $style .= $this->get_styles_to_grid_layout_7();
        } else if( $this->is_layout( 8 ) ){//7
            $style .= $this->get_styles_to_grid_layout_8();
        } else if( $this->is_layout( array( 9, 11 ) ) ){//8, 10
            $style .= $this->get_styles_to_grid_layout_9_11();
        } else if( $this->is_layout( array( 10, 12, 13 ) ) ){//9, 11, 12
            $style .= $this->get_styles_to_grid_layout_flip_cards();
        } else if( $this->is_layout( array( 14, 15 ) ) ){//13, 14
            $style .= $this->get_styles_to_grid_layout_14_15();
        }

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_styles_to_grid_layouts
    |---------------------------------------------------------------------------------------------------
    */
    public function get_styles_to_grid_layouts(){
        $style = '';
        $layout = $this->get_layout_options();

        //Bg [1, 2, 3, 4, 5, 6]
        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-figcaption" );
        if( $layout['bgGradientEnabled'] ){
            $css->prop( 'background', $this->get_grid_background( 1, true ) );
        } else{
            $css->prop( 'background', $this->get_grid_background( 1, false ) );
        }
        $style .= $css->build_css();

        if( $this->is_layout( array( 1 ) ) ){
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-active .tgr-figcaption, .tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner:hover .tgr-figcaption" );
            if( $layout['bgGradientEnabled2'] ){
                $css->prop( 'background', $this->get_grid_background( 2, true ) );
            } else{
                $css->prop( 'background', $this->get_grid_background( 2, false ) );
            }
            $style .= $css->build_css();

            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-image .tgr-item-desc" );
            if( $layout['bgGradientEnabled2'] ){
                $css->prop( 'background', $this->get_grid_background( 2, true ) );
            } else{
                $css->prop( 'background', $this->get_grid_background( 2, false ) );
            }
            $style .= $css->build_css();

        }
        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_styles_to_grid_layout_7
    |---------------------------------------------------------------------------------------------------
    */
    public function get_styles_to_grid_layout_7(){
        $style = '';
        $layout = $this->get_layout_options();

        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner" );
        if( $layout['bgGradientEnabled'] ){
            $css->prop( 'background', $this->get_grid_background( 1, true ) );
        } else{
            $css->prop( 'background', $this->get_grid_background( 1, false ) );
        }
        $style .= $css->build_css();

        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-figcaption:before" );
        if( $layout['bgGradientEnabled2'] ){
            $css->prop( 'background', $this->get_grid_background( 2, true ) );
        } else{
            $css->prop( 'background', $this->get_grid_background( 2, false ) );
        }
        $style .= $css->build_css();

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_styles_to_grid_layout_8
    |---------------------------------------------------------------------------------------------------
    */
    public function get_styles_to_grid_layout_8(){
        $style = '';
        $layout = $this->get_layout_options();

        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-figcaption" );
        if( $layout['bgGradientEnabled'] ){
            $css->prop( 'background', $this->get_grid_background( 1, true ) );
        } else{
            $css->prop( 'background', $this->get_grid_background( 1, false ) );
        }
        $style .= $css->build_css();

        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-social, .tgr-team-$this->id .tgr-layout-type-grid .tgr-item-desc" );
        if( $layout['bgGradientEnabled2'] ){
            $css->prop( 'background', $this->get_grid_background( 2, true ) );
        } else{
            $css->prop( 'background', $this->get_grid_background( 2, false ) );
        }
        $style .= $css->build_css();

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_styles_to_grid_layout_9_11
    |---------------------------------------------------------------------------------------------------
    */
    public function get_styles_to_grid_layout_9_11(){
        $style = '';
        $layout = $this->get_layout_options();

        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner, .tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner:hover .tgr-figcaption, .tgr-team-$this->id .tgr-layout-type-grid .tgr-grid-item.tgr-active .tgr-figcaption" );
        if( $layout['bgGradientEnabled'] ){
            $css->prop( 'background', $this->get_grid_background( 1, true ) );
        } else{
            $css->prop( 'background', $this->get_grid_background( 1, false ) );
        }
        $style .= $css->build_css();

        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-figcaption" );
        if( $layout['bgGradientEnabled'] ){
            $css->prop( 'background', "linear-gradient( {$layout['bgGradientAngle']} , rgba(255,255,255,0), {$layout['bgGradientColor']})" );
        } else{
            $css->prop( 'background', $this->get_grid_background( 1, false ) );
        }
        $style .= $css->build_css();

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_styles_to_grid_layout_flip_cards
    |---------------------------------------------------------------------------------------------------
    */
    public function get_styles_to_grid_layout_flip_cards(){
        $style = '';
        $layout = $this->get_layout_options();

        if( $this->is_layout( array( 13 ) ) ){//12
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-flip-back" );
            if( $layout['bgGradientEnabled'] ){
                $css->prop( 'background', $this->get_grid_background( 1, true ) );
            } else{
                $css->prop( 'background', $this->get_grid_background( 1, false ) );
            }
            $style .= $css->build_css();
        } else{
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-figure" );
            if( $layout['bgGradientEnabled'] ){
                $css->prop( 'background', $this->get_grid_background( 1, true ) );
            } else{
                $css->prop( 'background', $this->get_grid_background( 1, false ) );
            }
            $style .= $css->build_css();

            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-flip-back, .tgr-team-$this->id .tgr-layout-type-grid .tgr-item-inner:hover .tgr-figure, .tgr-team-$this->id .tgr-layout-type-grid .tgr-grid-item.tgr-active .tgr-figure, .tgr-team-$this->id .tgr-layout-type-grid .tgr-item-image .tgr-item-social" );
            if( $layout['bgGradientEnabled2'] ){
                $css->prop( 'background', $this->get_grid_background( 2, true ) );
            } else{
                $css->prop( 'background', $this->get_grid_background( 2, false ) );
            }
            $style .= $css->build_css();
        }

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_styles_to_grid_layout_14_15
    |---------------------------------------------------------------------------------------------------
    */
    public function get_styles_to_grid_layout_14_15(){
        $style = '';
        $layout = $this->get_layout_options();

        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-figcaption, .tgr-team-$this->id .tgr-layout-type-grid .tgr-item-image .tgr-item-header" );
        if( $layout['bgGradientEnabled'] ){
            $css->prop( 'background', $this->get_grid_background( 1, true ) );
        } else{
            $css->prop( 'background', $this->get_grid_background( 1, false ) );
        }
        $style .= $css->build_css();

        if( $this->is_layout( array( 15 ) ) ){//14
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-grid .tgr-figcaption" );
            $css->prop( 'top', CSS::number( $layout['borderWidth2'], 'px' ) );
            $css->prop( 'right', CSS::number( $layout['borderWidth2'], 'px' ) );
            $css->prop( 'bottom', CSS::number( $layout['borderWidth2'], 'px' ) );
            $css->prop( 'left', CSS::number( $layout['borderWidth2'], 'px' ) );
            $style .= $css->build_css();
        }

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_list_styles
    |---------------------------------------------------------------------------------------------------
    */
    public function get_list_styles(){
        $style = '';
        $layout = $this->get_layout_options();

        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-list .tgr-item" );
        if( $layout['bgGradientEnabled'] ){
            $css->prop( 'background', $this->get_grid_background( 1, true ) );
        } else{
            $css->prop( 'background', $this->get_grid_background( 1, false ) );
        }
        if( $layout['list']['borderBottomOnly'] ){
            $css->prop( 'border-width', CSS::number( $layout['borderWidth'], 'px' ) );
            $css->prop( 'border-color', $layout['borderColor'] );
        } else{
            $css->prop( 'border', CSS::number( $layout['borderWidth'], 'px' ) . " solid {$layout['borderColor']}" );
            $css->prop( 'margin-bottom', '40px' );
        }
        $style .= $css->build_css();

        return $style;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_table_styles
    |---------------------------------------------------------------------------------------------------
    */
    public function get_table_styles(){
        $style = '';
        $layout = $this->get_layout_options();

        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-table .tgr-table" );
        if( $layout['bgGradientEnabled'] ){
            $css->prop( 'background', $this->get_grid_background( 1, true ) );
        } else{
            $css->prop( 'background', $this->get_grid_background( 1, false ) );
        }
        $style .= $css->build_css();

        $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-table .tgr-table .tgr-th, .tgr-team-$this->id .tgr-layout-type-table .tgr-table .tgr-td" );
        $css->prop( 'border-width', CSS::number( $layout['borderWidth'], 'px' ) . ' 0' );
        $css->prop( 'border-color', $layout['borderColor'] );
        $style .= $css->build_css();

        if( $layout['table']['bordered'] ){
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-table .tgr-table.tgr-table-bordered .tgr-th, .tgr-team-$this->id .tgr-layout-type-table .tgr-table.tgr-table-bordered .tgr-td" );
            $css->prop( 'border-width', CSS::number( $layout['borderWidth'], 'px' ) );
            $css->prop( 'border-color', $layout['borderColor'] );
            $style .= $css->build_css();

            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-table .tgr-table.tgr-table-bordered .tgr-tr.tgr-item" );
            $css->prop( 'border-width', CSS::number( $layout['borderWidth'], 'px' ) );
            $css->prop( 'border-color', $layout['borderColor'] );
            $style .= $css->build_css(array(), '@media (max-width: '.TGR_BREAKPOINT_TABLET.'px)');
        }

        if( $layout['table']['striped'] ){
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-table .tgr-table.tgr-table-striped .tgr-th, .tgr-team-$this->id .tgr-layout-type-table .tgr-table.tgr-table-striped .tgr-tr:nth-child(odd)" );
            $css->prop( 'background', $layout['table']['stripedColor'] );
            $style .= $css->build_css();
        }
        if( $layout['table']['hover'] ){
            $css = new CSS( ".tgr-team-$this->id .tgr-layout-type-table .tgr-table.tgr-table-hover .tgr-tr.tgr-item:hover" );
            $css->prop( 'background', $layout['table']['hoverColor'] );
            $style .= $css->build_css();
        }

        return $style;
    }
}